@@ -7,7 +7,7 @@
 
 use std::marker::PhantomData;
 
-use nalgebra::Vector2;
+use nalgebra::Point2;
 
 use crate::{iterators::Slicer, Layer};
 
@@ -17,6 +17,7 @@ use crate::{iterators::Slicer, Layer};
 
 /// A [`Slicer`] which wrapps another [`Slicer`] and modifies it to produce the index of the item
 /// as well as the item itself.
+#[derive(Debug, Clone, Copy)]
 pub struct Indexed<'a, L, T, S>
 where
     L: Layer,
@@ -50,9 +51,9 @@ where
     L: Layer,
     S: Slicer<'a, L, T>,
 {
-    type Output = ((L, Vector2<usize>), S::Output);
+    type Output = ((L, Point2<usize>), S::Output);
 
-    type OutputMut = ((L, Vector2<usize>), S::OutputMut);
+    type OutputMut = ((L, Point2<usize>), S::OutputMut);
 
     fn slice(&self, data: &'a ndarray::Array2<T>) -> Option<Self::Output> {
         let item = self.slicer.slice(data)?;
@@ -70,14 +71,13 @@ where
         self.slicer.advance()
     }
 
-    fn index(&self) -> Option<nalgebra::Vector2<usize>> {
+    fn index(&self) -> Option<nalgebra::Point2<usize>> {
         self.slicer.index()
     }
 
     fn reset(&mut self, layer: Option<L>) {
-        match layer {
-            Some(ref l) => self.layer = l.clone(),
-            None => (),
+        if let Some(ref l) = layer {
+            self.layer = l.clone()
         }
 
         self.slicer.reset(layer)
