@@ -1,5 +1,44 @@
 //! Provides iterators over [`CellMap`]s.
 //!
+//! Iterators are constructed using the `iter` family of functions on [`CellMap`], such as
+//! [`CellMap::iter()`] and [`CellMap::window_iter_mut()`]. Which function you use determines
+//! which [`Slicer`] the iterator uses, in other words what order and shape the iterated items will
+//! be produced in.
+//!
+//! Once constructed both [`CellMapIter`] and [`CellMapIterMut`] provide functions to modify the
+//! which layers are produced, and whether or not the items also produce their indexes. These can
+//! be used like iterator combinators.
+//!
+//! # Examples
+//!
+//! Iterate over a 3x3 window of items in the `Height` layer, while also returning the indices
+//! of the central cell of the window:
+//!
+//! ```
+//! # use cell_map::{CellMap, CellMapParams, Layer};
+//! # use nalgebra::Vector2;
+//! # #[derive(Layer, Clone, Debug)]
+//! # enum MyLayer {
+//! #     Height,
+//! #     Gradient,
+//! #     Roughness
+//! # }
+//! #
+//! # // Creates a new 5x5 map where each cell is 1.0 units wide, which is centred on (0, 0), with
+//! # // all elements initialised to 1.0.
+//! # let mut map = CellMap::<MyLayer, f64>::new_from_elem(
+//! #     CellMapParams {
+//! #         cell_size: Vector2::new(1.0, 1.0),
+//! #         num_cells: Vector2::new(5, 5),
+//! #         ..Default::default()
+//! #     },
+//! #     1.0,
+//! # );
+//! for ((layer, index), height) in map.window_iter(Vector2::new(1, 1)).unwrap().indexed() {
+//!     println!("[{:?}, {}, {}] = {}", layer, index.x, index.y, height);
+//! }
+//! ```
+//!
 //! [`CellMap`]: crate::CellMap
 
 // ------------------------------------------------------------------------------------------------
@@ -8,6 +47,7 @@
 
 pub mod indexed;
 pub mod layerers;
+pub mod positioned;
 pub mod slicers;
 #[cfg(test)]
 mod tests;
@@ -17,18 +57,20 @@ mod tests;
 // ------------------------------------------------------------------------------------------------
 
 use layerers::*;
-use nalgebra::Vector2;
+use nalgebra::{Point2, Vector2};
 use slicers::*;
 
-use crate::{CellMap, CellMapError, Layer};
+use crate::{CellMap, Error, Layer};
 
-use self::indexed::Indexed;
+use self::{indexed::Indexed, positioned::Positioned};
 
 // ------------------------------------------------------------------------------------------------
 // STRUCTS
 // ------------------------------------------------------------------------------------------------
 
-/// A non-mutable iterator over a [`CellMap`], see [`Slicer`] and [`layerers`] for more information.
+/// A non-mutable iterator over a [`CellMap`], see [`Slicer`] and [`layerers`] for more
+/// information.
+#[derive(Debug, Clone, Copy)]
 pub struct CellMapIter<'m, L, T, R, S>
 where
     L: Layer,
@@ -41,6 +83,7 @@ where
 }
 
 /// A mutable iterator over a [`CellMap`], see [`Slicer`] and [`layerers`] for more information.
+#[derive(Debug)]
 pub struct CellMapIterMut<'m, L, T, R, S>
 where
     L: Layer,
@@ -75,7 +118,7 @@ where
     pub(crate) fn new_windows(
         map: &'m CellMap<L, T>,
         semi_width: Vector2<usize>,
-    ) -> Result<CellMapIter<'m, L, T, Many<L>, Windows>, CellMapError> {
+    ) -> Result<CellMapIter<'m, L, T, Many<L>, Windows>, Error> {
         Ok(CellMapIter {
             map,
             layerer: Many {
@@ -85,6 +128,20 @@ where
         })
     }
 
+    pub(crate) fn new_line(
+        map: &'m CellMap<L, T>,
+        start_position: Point2<f64>,
+        end_position: Point2<f64>,
+    ) -> Result<CellMapIter<'m, L, T, Many<L>, Line>, Error> {
+        Ok(CellMapIter {
+            map,
+            layerer: Many {
+                layers: L::all().into(),
+            },
+            slicer: Line::from_map::<L, T>(map.metadata, start_position, end_position)?,
+        })
+    }
+
     /// Converts this iterator to use a [`Single`] layerer, produing data from only one layer.
     pub fn layer(self, layer: L) -> CellMapIter<'m, L, T, Single<L>, S> {
         CellMapIter {
@@ -114,6 +171,17 @@ where
             slicer: Indexed::new(self.slicer, current_layer),
         }
     }
+
+    /// Converts this iterator to also produce the position of the iterated item as well as its
+    /// value.
+    pub fn positioned(self) -> CellMapIter<'m, L, T, R, Positioned<'m, L, T, S>> {
+        let current_layer = self.layerer.current().unwrap();
+        CellMapIter {
+            map: self.map,
+            layerer: self.layerer,
+            slicer: Positioned::new(self.slicer, current_layer, self.map.to_parent()),
+        }
+    }
 }
 
 impl<'m, L, T, R, S> CellMapIterMut<'m, L, T, R, S>
@@ -139,7 +207,7 @@ where
     pub(crate) fn new_windows(
         map: &'m mut CellMap<L, T>,
         semi_width: Vector2<usize>,
-    ) -> Result<CellMapIterMut<'m, L, T, Many<L>, Windows>, CellMapError> {
+    ) -> Result<CellMapIterMut<'m, L, T, Many<L>, Windows>, Error> {
         let slicer = Windows::from_map(map, semi_width)?;
 
         Ok(CellMapIterMut {
@@ -151,6 +219,21 @@ where
         })
     }
 
+    pub(crate) fn new_line(
+        map: &'m mut CellMap<L, T>,
+        start_position: Point2<f64>,
+        end_position: Point2<f64>,
+    ) -> Result<CellMapIterMut<'m, L, T, Many<L>, Line>, Error> {
+        let metadata = map.metadata;
+        Ok(CellMapIterMut {
+            map,
+            layerer: Many {
+                layers: L::all().into(),
+            },
+            slicer: Line::from_map::<L, T>(metadata, start_position, end_position)?,
+        })
+    }
+
     /// Converts this iterator to use a [`Single`] layerer, produing data from only one layer.
     pub fn layer(self, layer: L) -> CellMapIterMut<'m, L, T, Single<L>, S> {
         CellMapIterMut {
@@ -189,6 +272,18 @@ where
             slicer: Indexed::new(self.slicer, current_layer),
         }
     }
+
+    /// Converts this iterator to also produce the position of the iterated item as well as its
+    /// value.
+    pub fn positioned(self) -> CellMapIterMut<'m, L, T, R, Positioned<'m, L, T, S>> {
+        let current_layer = self.layerer.current().unwrap();
+        let to_parent = self.map.to_parent();
+        CellMapIterMut {
+            map: self.map,
+            layerer: self.layerer,
+            slicer: Positioned::new(self.slicer, current_layer, to_parent),
+        }
+    }
 }
 
 // ------------------------------------------------------------------------------------------------
