@@ -35,7 +35,7 @@
 //! To create a new map:
 //!
 //! ```rust
-//! use cell_map::{CellMap, CellMapParams};
+//! use cell_map::{CellMap, CellMapParams, Layer};
 //! use nalgebra::Vector2;
 //!
 //! # #[derive(Layer, Clone, Debug)]
@@ -44,12 +44,13 @@
 //! #     Gradient,
 //! #     Roughness
 //! # }
-//! // Creates a new 5x5 map where each cell is 1.0 units wide, which is centred on (0, 0).
+//! // Creates a new 5x5 map where each cell is 1.0 units wide, which is centred on (0, 0), with
+//! // all elements initialised to 1.0.
 //! let my_map = CellMap::<MyLayer, f64>::new_from_elem(
 //!     CellMapParams {
 //!         cell_size: Vector2::new(1.0, 1.0),
 //!         num_cells: Vector2::new(5, 5),
-//!         centre: Vector2::new(0.0, 0.0),
+//!         ..Default::default()
 //!     },
 //!     1.0,
 //! );
@@ -57,15 +58,16 @@
 //!
 //! ### Iterating Over Cells
 //!
-//! `CellMap` provides the following iteration types over its cells:
-//!   - `CellIter` - iterate each cell in the map
-//!   - `WindowIter` - iterate a window through the map
+//! [`CellMap`] provides methods to produce iterators over its data:
+//!   - [`CellMap::iter()`] gives an iterator over all cells in every layer of the map
+//!   - [`CellMap::window_iter()`] gives an iterator over rectangular windows into the map
+//!   - [`CellMap::line_iter()`] gives an iterator of cells between two world points
 //!
 //! All iterators also provide a mutable variant, and more iterators are planned
 //! in the future!
 //!
-//! You can modify iterators so they produce `Layered` or `Indexed` iterators as
-//! well.
+//! You can modify iterators so they produce their indexes, as well as controlling which layers the
+//! data comes from. See [`iterators`] for more information.
 //!
 //! ```rust
 //! # use cell_map::{CellMap, CellMapParams, Layer};
@@ -78,12 +80,13 @@
 //! #     Roughness
 //! # }
 //! #
-//! # // Creates a new 5x5 map where each cell is 1.0 units wide, which is centred on (0, 0).
+//! # // Creates a new 5x5 map where each cell is 1.0 units wide, which is centred on (0, 0), with
+//! # // all elements initialised to 1.0.
 //! # let mut my_map = CellMap::<MyLayer, f64>::new_from_elem(
 //! #     CellMapParams {
 //! #         cell_size: Vector2::new(1.0, 1.0),
 //! #         num_cells: Vector2::new(5, 5),
-//! #         centre: Vector2::new(0.0, 0.0),
+//! #         ..Default::default()
 //! #     },
 //! #     1.0,
 //! # );
@@ -100,7 +103,7 @@
 //!
 //! // Check that our map is how we expect it
 //! for ((layer, cell), &value) in my_map.iter().indexed() {
-//!     if let MyLayer::Roughness = layer {
+//!     if matches!(layer, MyLayer::Roughness) {
 //!         assert_eq!(value, 0.0);
 //!     }
 //!     else if cell.x == 0 || cell.x == 4 || cell.y == 0 || cell.y == 4 {
@@ -113,16 +116,25 @@
 //! ```
 
 #![warn(missing_docs)]
+#![warn(missing_copy_implementations)]
+#![warn(missing_debug_implementations)]
 
 // ------------------------------------------------------------------------------------------------
 // MODULES
 // ------------------------------------------------------------------------------------------------
 
-mod cell_map;
+#[macro_use]
+mod macros;
+
+pub(crate) mod cell_map;
+pub mod cell_map_file;
 pub mod error;
 pub(crate) mod extensions;
 pub mod iterators;
 mod layer;
+mod map_metadata;
+#[cfg(test)]
+mod tests;
 
 // ------------------------------------------------------------------------------------------------
 // EXPORTS
@@ -130,18 +142,38 @@ mod layer;
 
 pub use crate::cell_map::{CellMap, CellMapParams};
 pub use cell_map_macro::Layer;
-pub use error::CellMapError;
+pub use error::Error;
 pub use layer::Layer;
 
 // ------------------------------------------------------------------------------------------------
 // USEFUL TEST UTILITIES
 // ------------------------------------------------------------------------------------------------
 
+#[cfg(feature = "debug_maps")]
+use serde::Serialize;
+
+/// Writes the given map to the given location, prepending "_debug_" to the name.
+#[cfg(feature = "debug_maps")]
+pub fn write_debug_map<L: Layer + Serialize, T: Serialize + Clone>(
+    map: &CellMap<L, T>,
+    name: &str,
+) {
+    #[cfg(feature = "debug_maps")]
+    {
+        let file_name = &format!("_debug_{}_map.json", name);
+        map.write_json(file_name)
+            .expect("Failed to write debug map!");
+    }
+}
 #[cfg(test)]
-mod test_utils {
+#[macro_use]
+pub(crate) mod test_utils {
+
+    use serde::Serialize;
+
     use crate::Layer;
 
-    #[derive(Clone, Copy, Debug)]
+    #[derive(Clone, Copy, Debug, Serialize)]
     #[allow(dead_code)]
     pub enum TestLayers {
         Layer0,
